<?php

namespace OroCRM\Bundle\MagentoBundle\Tests\Unit\Provider;

use Akeneo\Bundle\BatchBundle\Entity\JobExecution;
use Akeneo\Bundle\BatchBundle\Entity\StepExecution;
use Akeneo\Bundle\BatchBundle\Item\ExecutionContext;

use Doctrine\Common\Persistence\ManagerRegistry;

use Symfony\Component\HttpKernel\Log\NullLogger;

use Oro\Bundle\IntegrationBundle\Entity\Channel;
use Oro\Bundle\IntegrationBundle\Entity\Status;
use Oro\Bundle\IntegrationBundle\Entity\Repository\ChannelRepository;
use Oro\Bundle\IntegrationBundle\Entity\Transport;
use Oro\Bundle\IntegrationBundle\Logger\LoggerStrategy;
use Oro\Bundle\ImportExportBundle\Context\Context;
use Oro\Bundle\ImportExportBundle\Context\ContextRegistry;
use Oro\Bundle\IntegrationBundle\Provider\ConnectorContextMediator;
use Oro\Bundle\IntegrationBundle\Provider\ConnectorInterface;

use OroCRM\Bundle\MagentoBundle\Provider\AbstractMagentoConnector;
use OroCRM\Bundle\MagentoBundle\Provider\Transport\MagentoTransportInterface;

abstract class MagentoConnectorTestCase extends \PHPUnit_Framework_TestCase
{
    /** @var MagentoTransportInterface|\PHPUnit_Framework_MockObject_MockObject */
    protected $transportMock;

    /** @var StepExecution|\PHPUnit_Framework_MockObject_MockObject */
    protected $stepExecutionMock;

    /** @var ManagerRegistry|\PHPUnit_Framework_MockObject_MockObject */
    protected $managerRegistryMock;

    /** @var ChannelRepository|\PHPUnit_Framework_MockObject_MockObject */
    protected $integrationRepositoryMock;

    /** @var JobExecution|\PHPUnit_Framework_MockObject_MockObject */
    protected $jobExecutionMock;

    /** @var ExecutionContext|\PHPUnit_Framework_MockObject_MockObject */
    protected $executionContextMock;

    /** @var array */
    protected $config = [
        'sync_settings' => ['mistiming_assumption_interval' => '2 minutes']
    ];

    protected function setUp()
    {
        $this->transportMock     = $this
            ->getMock('OroCRM\\Bundle\\MagentoBundle\\Provider\\Transport\\MagentoTransportInterface');

        $this->stepExecutionMock = $this->getMockBuilder('Akeneo\\Bundle\\BatchBundle\\Entity\\StepExecution')
            ->setMethods(['getExecutionContext', 'getJobExecution'])
            ->disableOriginalConstructor()->getMock();

        $this->executionContextMock = $this->getMock('Akeneo\Bundle\BatchBundle\Item\ExecutionContext');

        $this->jobExecutionMock = $this->getMock('Akeneo\Bundle\BatchBundle\Entity\JobExecution');
        $this->jobExecutionMock->expects($this->any())
            ->method('getExecutionContext')
            ->will($this->returnValue($this->executionContextMock));

        $this->stepExecutionMock->expects($this->any())
            ->method('getJobExecution')
            ->will($this->returnValue($this->jobExecutionMock));

        $this->managerRegistryMock = $this->getMockBuilder('Doctrine\\Common\\Persistence\\ManagerRegistry')
            ->disableOriginalConstructor()
            ->getMock();

        $this->integrationRepositoryMock = $this->getMockBuilder(
            'Oro\\Bundle\\IntegrationBundle\\Entity\\Repository\\ChannelRepository'
        )
            ->disableOriginalConstructor()
            ->getMock();

        $this->managerRegistryMock->expects($this->any())
            ->method('getRepository')
            ->with('OroIntegrationBundle:Channel')
            ->will($this->returnValue($this->integrationRepositoryMock));
    }

    public function tearDown()
    {
        unset($this->transportMock, $this->stepExecutionMock);
    }

    public function testInitialization()
    {
        $connector = $this->getConnector($this->transportMock, $this->stepExecutionMock);
        $this->transportMock->expects($this->at(0))->method($this->getIteratorGetterMethodName())
            ->will($this->returnValue($this->getMock('\Iterator')));
        $connector->setStepExecution($this->stepExecutionMock);
    }

    public function testInitializationInUpdatedMode()
    {
        $channel   = new Channel();
        $connector = $this->getConnector($this->transportMock, $this->stepExecutionMock, $channel);

        $status = new Status();
        $status->setCode($status::STATUS_COMPLETED);
        $status->setConnector($connector->getType());
        $status->setDate(new \DateTime('-10 days', new \DateTimeZone('UTC')));

        $this->expectLastCompletedStatusForConnector($status, $channel, $connector->getType());

        $expectedDateInFilter = clone $status->getDate();
        $assumptionInterval   = $this->config['sync_settings']['mistiming_assumption_interval'];
        $expectedDateInFilter->sub(\DateInterval::createFromDateString($assumptionInterval));

        $iterator = $this->getMock('OroCRM\\Bundle\\MagentoBundle\\Provider\\Iterator\\UpdatedLoaderInterface');
        $iterator->expects($this->once())->method('setStartDate')->with($this->equalTo($expectedDateInFilter));
        $this->transportMock->expects($this->at(0))->method($this->getIteratorGetterMethodName())
            ->will($this->returnValue($iterator));

        $connector->setStepExecution($this->stepExecutionMock);
    }

    /**
     * @dataProvider predefinedIteratorProvider
     *
     * @param mixed $iterator
     * @param null  $exceptionExpected
     */
    public function testInitializationWithPredefinedFilters($iterator, $exceptionExpected = null)
    {
        if (null !== $exceptionExpected) {
            $this->setExpectedException($exceptionExpected);
        } else {
            $iterator->expects($this->once())->method('setPredefinedFiltersBag');
        }
        $context = new Context(['filters' => ['test' => 1]]);

        $connector = $this->getConnector($this->transportMock, $this->stepExecutionMock, null, $context);

        $this->transportMock->expects($this->at(0))->method($this->getIteratorGetterMethodName())
            ->will($this->returnValue($iterator));

        $connector->setStepExecution($this->stepExecutionMock);
    }

    /**
     * @return array
     */
    public function predefinedIteratorProvider()
    {
        $iterator1 = $this->getMock('\Iterator');
        $iterator2 = $this->getMock('OroCRM\Bundle\MagentoBundle\Tests\Unit\Fixtures\PredefinedFiltersAwareFixture');

        return [
            'should throw exception' => [
                $iterator1,
                '\LogicException'
            ],
            'should process filters' => [
                $iterator2
            ]
        ];
    }

    /**
     * @expectedException \LogicException
     */
    public function testInitializationErrors()
    {
        $connector = $this->getConnector(null, $this->stepExecutionMock);
        $this->transportMock->expects($this->never())->method('init');

        $connector->setStepExecution($this->stepExecutionMock);
    }

    /**
     * @expectedException \LogicException
     * @expectedExceptionMessage Option "transport" should implement "MagentoTransportInterface"
     */
    public function testInitializationErrorsBadTransportGiven()
    {
        $badTransport = $this->getMock('Oro\\Bundle\\IntegrationBundle\\Provider\\TransportInterface');
        $connector    = $this->getConnector($badTransport, $this->stepExecutionMock);
        $this->transportMock->expects($this->never())->method('init');

        $connector->setStepExecution($this->stepExecutionMock);
    }

    /**
     * @dataProvider readItemDatesDataProvider
     *
     * @param string  $dateInContext
     * @param string  $dateInItem
     * @param string  $expectedDate
     * @param boolean $hasData
     * @param string  $dateInIterator
     */
    public function testRead($dateInContext, $dateInItem, $expectedDate, $hasData = true, $dateInIterator = null)
    {
        $iteratorMock = $this->getMock('OroCRM\\Bundle\\MagentoBundle\\Provider\\Iterator\\UpdatedLoaderInterface');

        $connector = $this->getConnector($this->transportMock, $this->stepExecutionMock);

        $this->transportMock->expects($this->at(0))->method($this->getIteratorGetterMethodName())
            ->will($this->returnValue($iteratorMock));

        $connector->setStepExecution($this->stepExecutionMock);
        $context = $this->stepExecutionMock->getExecutionContext();
        $context->put(ConnectorInterface::CONTEXT_CONNECTOR_DATA_KEY, ['lastSyncItemDate' => $dateInContext]);

        $testValue = [
            'created_at' => '01.01.2200 14:15:08',
            'updatedAt'  => $dateInItem
        ];

        if ($hasData) {
            $context->put(ConnectorInterface::CONTEXT_CONNECTOR_DATA_KEY, ['lastSyncItemDate' => $dateInContext]);

            $iteratorMock->expects($this->once())->method('rewind');
            $iteratorMock->expects($this->once())->method('next');
            $iteratorMock->expects($this->any())->method('valid')->will($this->onConsecutiveCalls(true, false));
            $iteratorMock->expects($this->once())->method('current')->will($this->returnValue($testValue));

            $this->assertEquals($testValue, $connector->read());
        } else {
            $context->put(ConnectorInterface::CONTEXT_CONNECTOR_DATA_KEY, ['lastSyncItemDate' => $dateInIterator]);

            $iteratorMock->expects($this->once())->method('rewind');
            $iteratorMock->expects($this->never())->method('next');
            $iteratorMock->expects($this->any())->method('valid')->will($this->returnValue(false));
            $iteratorMock->expects($this->never())->method('current')->will($this->returnValue(null));
            $iteratorMock->expects($this->at(0))->method('getStartDate')
                ->will($this->returnValue(new \Datetime($dateInIterator)));
            $iteratorMock->expects($this->at(1))->method('getStartDate')->will($this->returnValue($dateInIterator));
        }

        $this->assertNull($connector->read());

        $connectorData = $context->get(ConnectorInterface::CONTEXT_CONNECTOR_DATA_KEY);

        $this->assertArrayHasKey('lastSyncItemDate', $connectorData);

        if ($hasData) {
            $this->assertSame($expectedDate, $connectorData['lastSyncItemDate']);
        } else {
            $this->assertSame($dateInIterator, $connectorData['lastSyncItemDate']);
        }
    }

    /**
     * @return array
     */
    public function readItemDatesDataProvider()
    {
        return [
            'empty context, should take updated date from item'                 => [
                '$dateInContext' => null,
                '$dateInItem'    => '01.01.2000 14:15:08',
                '$expectedDate'  => '01.01.2000 14:15:08',
            ],
            'date in context given but empty date in item, should not override' => [
                '$dateInContext' => '01.01.2000 14:15:08',
                '$dateInItem'    => null,
                '$expectedDate'  => '01.01.2000 14:15:08',
            ],
            'should take greater from item'                                     => [
                '$dateInContext' => '01.01.2000 14:15:08',
                '$dateInItem'    => '01.02.2000 14:15:08',
                '$expectedDate'  => '01.02.2000 14:15:08',
            ],
            'should take greater from context'                                  => [
                '$dateInContext' => '01.01.2001 14:15:08',
                '$dateInItem'    => '01.01.2000 14:15:08',
                '$expectedDate'  => '01.01.2001 14:15:08',
            ],
            'without data'                                                      => [
                '$dateInContext' => null,
                '$dateInItem'    => null,
                '$expectedDate'  => null,
                false,
                '01.01.2010 14:15:08',
            ],
        ];
    }

    /**
     * @param mixed        $transport
     * @param mixed        $stepExecutionMock
     * @param null|Channel $channel
     *
     * @param null         $context
     *
     * @return AbstractMagentoConnector
     */
    protected function getConnector($transport, $stepExecutionMock, $channel = null, $context = null)
    {
        /** @var \PHPUnit_Framework_MockObject_MockObject|ContextRegistry $contextRegistryMock */
        $contextRegistryMock = $this->getMock('Oro\Bundle\ImportExportBundle\Context\ContextRegistry');

        /** @var \PHPUnit_Framework_MockObject_MockObject|ConnectorContextMediator $contextMediatorMock */
        $contextMediatorMock = $this
            ->getMockBuilder('Oro\\Bundle\\IntegrationBundle\\Provider\\ConnectorContextMediator')
            ->disableOriginalConstructor()->getMock();

        /** @var \PHPUnit_Framework_MockObject_MockObject|Transport $transportSettings */
        $transportSettings = $this->getMockForAbstractClass('Oro\\Bundle\\IntegrationBundle\\Entity\\Transport');
        $channel           = $channel ? : new Channel();
        $channel->setTransport($transportSettings);

        $contextMock = $context ? : new Context([]);

        $executionContext = new ExecutionContext();
        $stepExecutionMock->expects($this->any())
            ->method('getExecutionContext')->will($this->returnValue($executionContext));

        $contextRegistryMock->expects($this->any())->method('getByStepExecution')
            ->will($this->returnValue($contextMock));
        $contextMediatorMock->expects($this->any())
            ->method('getInitializedTransport')->with($this->equalTo($channel))
            ->will($this->returnValue($transport));
        $contextMediatorMock->expects($this->any())
            ->method('getChannel')->with($this->equalTo($contextMock))
            ->will($this->returnValue($channel));

        $logger = new LoggerStrategy(new NullLogger());

        $connector = $this->getConnectorInstance($contextRegistryMock, $logger, $contextMediatorMock);
        $connector->setManagerRegistry($this->managerRegistryMock);

        return $connector;
    }

    /**
     * @param Status $expectedStatus
     * @param Channel $channel
     * @param string $connector
     */
    protected function expectLastCompletedStatusForConnector($expectedStatus, $channel, $connector)
    {
        $this->integrationRepositoryMock->expects($this->once())
            ->method('getLastStatusForConnector')
            ->with($channel, $connector, Status::STATUS_COMPLETED)
            ->will($this->returnValue($expectedStatus));
    }

    /**
     * @return bool
     */
    protected function supportsForceMode()
    {
        return false;
    }

    /**
     * @param ContextRegistry          $contextRegistry
     * @param LoggerStrategy           $logger
     * @param ConnectorContextMediator $contextMediator
     *
     * @return AbstractMagentoConnector
     */
    abstract protected function getConnectorInstance(
        ContextRegistry $contextRegistry,
        LoggerStrategy $logger,
        ConnectorContextMediator $contextMediator
    );

    /**
     * @return string
     */
    abstract protected function getIteratorGetterMethodName();
}
